import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class CompPanel extends JPanel implements ComponentListener, KeyListener, MouseListener, MouseMotionListener, ActionListener {

    final int COLS;
    final int ROWS;

    MegaLaevandus shipData;
    //The ship that has been chosen as "target" to place somewhere...
    Ship ship;
    //And lets remember the button too, where this ship has been taken from
    ShipButton button;

    Point cursor = new Point(0,0);
    int x, y, xErr, yErr, rectWidth, rectHeight, curWidth, curHeight;
    int width, height;
    BufferedImage bi;
    boolean mouseOver;

    public CompPanel(MegaLaevandus shipData){

	this.shipData=shipData;    
	//COLS=10; ROWS=10;
	ROWS=shipData.getRows();
	COLS=shipData.getCols();
	addMouseMotionListener(this);
	addMouseListener(this);
	addComponentListener(this);
	addKeyListener(this);
    }

    public void updateBufImage() {
	bi=new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
	Graphics g2 = bi.createGraphics();
	for(int n=0; n<ROWS; n++) {
	    for(int m=0; m<COLS; m++){
		g2.setColor(Color.black);
		g2.drawRect(m*rectWidth+xErr, n*rectHeight+yErr, rectWidth-1, rectHeight-1);
		//System.out.println("asked "+m+", "+n);
		if(shipData.getContents(m,n)>0)
		    g2.setColor(Color.yellow);
		else
		    g2.setColor(Color.blue);

		g2.fillRect(m*rectWidth+xErr+1, n*rectHeight+yErr+1, rectWidth-2, rectHeight-2);
	    }
	}
    }
/**
* called by swing when It needs redrawing.
* 
*/
    public void paintComponent(Graphics g){
	super.paintComponent(g);
	//Lets see.. what do we have to paint inside. There is the buffered image of the contents...
	g.drawImage(bi, 0, 0, this);
	//...and the cursor if mouse is over the panel and if there is ship to place
	//If there is a target(ship) to place, then lets draw a cursor for it.
	if(ship!=null){

	    if(cursor.x+ship.getWidth()>COLS)
		cursor.x=COLS-ship.getWidth();
	
	    if(cursor.y+ship.getHeight()>ROWS)
		cursor.y=ROWS-ship.getHeight();

	    //As we are drawing the cursor, first we need some guidelines, what color to paint the rect of cursor
	    //Lets ask it here, from the location where shipData points
	    //mouseMan little bit buggy right now.. so dont uncomment it :)
	    boolean[][] guide = shipData.mouseMan(ship, cursor.x, cursor.y);

	    if( hasFocus() || mouseOver ){
		for(int m=-1; m<=ship.getWidth(); m++){
		    for(int n=-1; n<=ship.getHeight(); n++){
		    	if(guide[n+1][m+1])
			    g.setColor(Color.green);
			else
			    g.setColor(Color.red);

			if(((cursor.x+m>=0)&&(cursor.x+m<COLS))&&((cursor.y+n>=0)&&(cursor.y+n<ROWS))){
			    g.drawRect((cursor.x+m)*rectWidth+xErr+1, (cursor.y+n)*rectHeight+yErr+1, rectWidth-3, rectHeight-3);
			    if(( m>=0 && m<ship.getWidth() )&&( n>=0 && n<ship.getHeight() )){
				g.fillRect((cursor.x+m)*rectWidth+xErr+2, (cursor.y+n)*rectHeight+yErr+2, rectWidth-4, rectHeight-4);
			    }
			}
		    }
		}
	    }
	}
    }

    //Method for changeing the ship - gets called when Player chooses a ship.
    public void setShip(Ship ship){
	this.ship=ship;
    }

    //Componentlisteners stuff...
    public void componentHidden(ComponentEvent e){System.out.println("Hidden");}
    public void componentMoved(ComponentEvent e){System.out.println("Moved");}
    public void componentResized(ComponentEvent e){
	//Set the neccessary variables
	System.out.println("Resized: w"+getWidth()+", h"+getHeight());
	width = getWidth();
	height = getHeight();
	xErr=(width%COLS)/2;
	yErr=(height%COLS)/2;
	rectWidth = width/COLS;
	rectHeight = height/ROWS;
	if(ship!=null){
	    curWidth = rectWidth*(ship.getWidth()+2);
	    curHeight = rectHeight*(ship.getHeight()+2);
	}

	//And update the drawing..
	updateBufImage();
	repaint();
    }
    public void componentShown(ComponentEvent e){System.out.println("shown");}

    public boolean isFocusTraversable(){
	return(true);
    }

    //KeyListener...
    public void keyTyped(KeyEvent e){}
    public void keyPressed(KeyEvent e){
	int keyCode = e.getKeyCode();
	//System.out.println(keyCode);
	if(ship!=null){
	    switch (keyCode) {
	    case 37: //leftArrow pressed
		if (cursor.x>0){
		    cursor.x--;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth+rectWidth, curHeight);
		}
		break;
	    case 38: //upArrow pressed
		if (cursor.y>0){
		    cursor.y--;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight+rectHeight);
		}
		break;
	    case 39: //rightArrow
		if (cursor.x<COLS-ship.getWidth()){
		    cursor.x++;
		    repaint((cursor.x-2)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth+rectWidth, curHeight);
		}
		break;
	    case 40: //downArrow
		if (cursor.y<ROWS-ship.getHeight()){
		    cursor.y++;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-2)*rectHeight + yErr, curWidth, curHeight+rectHeight);
		}
		break;
	    }
	}
    }

    public void keyReleased(KeyEvent e){}

    //mouseListening stuff comes from here..
    public void mousePressed(MouseEvent e){}
    public void mouseDragged(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}
    public void mouseMoved(MouseEvent e){
	x = e.getX()-xErr;
	y = e.getY()-yErr;
	if (rectWidth>0 && rectHeight>0) {
	    int newx = x/rectWidth;
	    int newy = y/rectHeight;
	    if (((Math.abs(newx - cursor.x) > 0) || (Math.abs(newy - cursor.y) > 0)) ){
		repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
		if( newx>=0 && newx<COLS ) 
		    cursor.x = newx;
		else 
		    cursor.x = (newx<0) ? 0 : COLS-1;

		if( newy>=0 && newy<ROWS)
		    cursor.y = newy;
		else
		    cursor.y = (newy<0) ? 0 : ROWS-1;

		//System.out.println("Cursor: " + cursor.x + ", " + cursor.y);
		repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
	    }
	}
    }

    public void mouseClicked(MouseEvent e){
	requestFocus();
	if(ship!=null){
	    if( (e.getModifiers() & InputEvent.BUTTON1_MASK)==InputEvent.BUTTON1_MASK ){
		if(shipData.asetaLaev(ship, cursor.x, cursor.y)>0){
		    button.shipPlaced();
		    ship=null;
		    button=null;
		    updateBufImage();
		    repaint();
		}
	    }else{
		ship.rotate();
		//Some stupid codec too (like the rest of the code should be less stupid)...
		curWidth = rectWidth*(ship.getWidth()+2);
		curHeight = rectHeight*(ship.getHeight()+2);
		//--end of that stupid part
		repaint();
	    }
	}
    }

    public void mouseEntered(MouseEvent e){
	//requestFocus();
	mouseOver=true;
	repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
    }

    public void mouseExited(MouseEvent e){
	mouseOver=false;
	repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
    }

    //ActionListener
    //For listening the instances of ShipButton class but there is no error (exception) code yet
    public void actionPerformed(ActionEvent e){
	System.out.println("Caught on Action!!");
	button = (ShipButton) e.getSource();
	this.ship=button.getShip();
	curWidth = rectWidth*(ship.getWidth()+2);
	curHeight = rectHeight*(ship.getHeight()+2);
	repaint();
    }
}
