class ShootingPanel extends JPanel implements ComponentListener, KeyListener, FocusListener, MouseListener, MouseMotionListener {

    final int COLS=10;
    final int ROWS=10;

    Ship ship=new Ship(1,4);
    Point cursor = new Point(0,0);
    int x, y, xErr, yErr, rectWidth, rectHeight, curWidth, curHeight;
    int width, height;
    BufferedImage bi;
    boolean mouseOver, hasFocus;

    public SuhtlevPaneel(){
    
	addMouseMotionListener(this);
	addMouseListener(this);
	addComponentListener(this);
	addKeyListener(this);
	addFocusListener(this);
    }

    public void updateBufImage() {
	bi=new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
	Graphics g2 = bi.createGraphics();
	for(int n=0; n<COLS; n++) {
	    for(int m=0; m<ROWS; m++){
		g2.setColor(Color.black);
		g2.drawRect(n*rectWidth+xErr, m*rectHeight+yErr, rectWidth-1, rectHeight-1);
		g2.setColor(Color.blue);
		g2.fillRect(n*rectWidth+xErr+1, m*rectHeight+yErr+1, rectWidth-2, rectHeight-2);
	    }
	}
    }

    public void paintComponent(Graphics g){
	super.paintComponent(g);
	//Lets see.. what do we have to paint inside. There is the buffered image of the contents...
	g.drawImage(bi, 0, 0, this);
	//...and the cursor if mouse is over the panel
	if(ship!=null){
	    if(cursor.x+ship.width>COLS)
		cursor.x=COLS-ship.width;
	
	    if(cursor.y+ship.height>ROWS)
		cursor.y=ROWS-ship.height;
	
	    if( hasFocus || mouseOver ){
		for(int m=-1; m<=ship.width; m++){
		    for(int n=-1; n<=ship.height; n++){
			g.setColor(Color.green);

			if(((cursor.x+m>=0)&&(cursor.x+m<COLS))&&((cursor.y+n>=0)&&(cursor.y+n<ROWS))){
			    g.drawRect((cursor.x+m)*rectWidth+xErr+1, (cursor.y+n)*rectHeight+yErr+1, rectWidth-3, rectHeight-3);
			    if(( m>=0 && m<ship.width )&&( n>=0 && n<ship.height )){
				g.fillRect((cursor.x+m)*rectWidth+xErr+2, (cursor.y+n)*rectHeight+yErr+2, rectWidth-4, rectHeight-4);
			    }
			}  
		    }
		}
	    }
	}
    }

    //Componentlisteners stuff...
    public void componentHidden(ComponentEvent e){}
    public void componentMoved(ComponentEvent e){}
    public void componentResized(ComponentEvent e){
	//Set the neccessary variables
	width = getWidth();
	height = getHeight();
	xErr=(width%COLS)/2;
	yErr=(height%COLS)/2;
	rectWidth = width/COLS;
	rectHeight = height/ROWS;
	if(ship!=null){
	    curWidth = rectWidth*(ship.width+2);
	    curHeight = rectHeight*(ship.height+2);
	}

	//And update the drawing..
	updateBufImage();	
    }

    public void componentShown(ComponentEvent e){}

    //To listen keys.. it would be nice to override this method...
    public boolean isFocusTraversable(){
	return(true);
    }

    //FocusListener
    public void focusGained(FocusEvent e){
	hasFocus = true;
	repaint();
    }
    public void focusLost(FocusEvent e){
	hasFocus = false;
	repaint();
    }

    //KeyListener...
    public void keyTyped(KeyEvent e){}
    public void keyPressed(KeyEvent e){
	int keyCode = e.getKeyCode();
	//System.out.println(keyCode);
	if(ship!=null){
	    switch (keyCode) {
	    case 37: //leftArrow pressed
		if (cursor.x>0){
		    cursor.x--;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth+rectWidth, curHeight);
		}
		break;
	    case 38: //upArrow pressed
		if (cursor.y>0){
		    cursor.y--;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight+rectHeight);
		}
		break;
	    case 39: //rightArrow
		if (cursor.x<COLS-ship.width){
		    cursor.x++;
		    repaint((cursor.x-2)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth+rectWidth, curHeight);
		}
		break;
	    case 40: //downArrow
		if (cursor.y<ROWS-ship.height){
		    cursor.y++;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-2)*rectHeight + yErr, curWidth, curHeight+rectHeight);
		}
		break;
	    }
	}
    }

    public void keyReleased(KeyEvent e){}

    //mouseListening stuff comes from here..
    public void mousePressed(MouseEvent e){}
    public void mouseDragged(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}
    public void mouseMoved(MouseEvent e){
	x = e.getX()-xErr;
	y = e.getY()-yErr;
	if (rectWidth>0 && rectHeight>0) {
	    int newx = x/rectWidth;
	    int newy = y/rectHeight;
	    if (((Math.abs(newx - cursor.x) > 0) || (Math.abs(newy - cursor.y) > 0)) ){
		repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
		if( newx>=0 && newx<COLS ) 
		    cursor.x = newx;
		else 
		    cursor.x = (newx<0) ? 0 : COLS-1;

		if( newy>=0 && newy<ROWS)
		    cursor.y = newy;
		else
		    cursor.y = (newy<0) ? 0 : ROWS-1;

		repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
	    }
	}
    }

    public void mouseClicked(MouseEvent e){
	requestFocus();
	if(ship!=null){
	    System.out.println(e.getModifiers());
	    if( (e.getModifiers() & InputEvent.BUTTON3_MASK)==InputEvent.BUTTON3_MASK ){
		System.out.println("Button 3");
	    }
	}
    }

    public void mouseEntered(MouseEvent e){
	//requestFocus();
	mouseOver=true;
	repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
    }

    public void mouseExited(MouseEvent e){
	mouseOver=false;
	repaint((cursor.x-1)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, curWidth, curHeight);
    }

}
