import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class ShootingPanel extends JPanel implements ComponentListener, KeyListener, MouseListener, MouseMotionListener, ActionListener {

    final int COLS;
    final int ROWS;

    MegaLaevandus shipData;

    Point cursor = new Point(0,0);
    int x, y, xErr, yErr, rectWidth, rectHeight;
    int width, height;
    BufferedImage bi;
    boolean mouseOver, timeToShoot = true;
    Lollus kala;

    public ShootingPanel(MegaLaevandus shipData){
	this.kala = new Lollus();
	this.shipData=shipData;    
	//COLS=10; ROWS=10;
	ROWS=shipData.getRows();
	COLS=shipData.getCols();
	addMouseMotionListener(this);
	addMouseListener(this);
	addComponentListener(this);
	addKeyListener(this);
    }

    public void updateBufImage() {
	bi=new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
	Graphics g2 = bi.createGraphics();
	for(int n=0; n<ROWS; n++) {
	    for(int m=0; m<COLS; m++){
		g2.setColor(Color.black);
		g2.drawRect(m*rectWidth+xErr, n*rectHeight+yErr, rectWidth-1, rectHeight-1);
		//System.out.println("asked "+m+", "+n);
		if(shipData.getContents(m,n)>0){
		    if(shipData.isBombarded(m,n))
		        g2.setColor(new Color(255, 200, 0));
		    else
		    	//Egg.. just set it to make yellow color.. to see enemy ships :)
		        g2.setColor(Color.blue);
		}else{
		    g2.setColor(Color.blue);
		}
		g2.fillRect(m*rectWidth+xErr+1, n*rectHeight+yErr+1, rectWidth-2, rectHeight-2);
		
	        if(shipData.isBombarded(m,n)){
		    g2.setColor(Color.white);
		    g2.drawLine(m*rectWidth+xErr, n*rectHeight+yErr, m*rectWidth+xErr+rectWidth, n*rectHeight+yErr+rectHeight);
		    g2.drawLine(m*rectWidth+xErr, n*rectHeight+yErr+rectHeight, m*rectWidth+xErr+rectWidth, n*rectHeight+yErr);
		 }
	    }
	}
    }
/**
* called by swing when It needs redrawing.
* 
*/
    public void paintComponent(Graphics g){
	super.paintComponent(g);
	//Lets see.. what do we have to paint inside. There is the buffered image of the contents...
	if(bi != null)
	   g.drawImage(bi, 0, 0, this);
	//.. and the cursor
	g.setColor(Color.green);
	g.drawRect(cursor.x*rectWidth+xErr+1, cursor.y*rectHeight+yErr+1, rectWidth-2, rectHeight-2);
    }

    //Componentlisteners stuff...
    public void componentHidden(ComponentEvent e){System.out.println("Hidden");}
    public void componentMoved(ComponentEvent e){System.out.println("Moved");}
    public void componentResized(ComponentEvent e){
	//Set the neccessary variables
	System.out.println("Resized: w"+getWidth()+", h"+getHeight());
	width = getWidth();
	height = getHeight();
	xErr=(width%COLS)/2;
	yErr=(height%COLS)/2;
	rectWidth = width/COLS;
	rectHeight = height/ROWS;

	//And update the drawing..
	updateBufImage();
	repaint();
    }
    public void componentShown(ComponentEvent e){System.out.println("shown");}

    public boolean isFocusTraversable(){
	return(true);
    }

    //KeyListener...
    public void keyTyped(KeyEvent e){}
    public void keyPressed(KeyEvent e){
	int keyCode = e.getKeyCode();
	//System.out.println(keyCode);
	switch (keyCode) {
	    case 37: //leftArrow pressed
		if (cursor.x>0){
		    cursor.x--;
		    repaint((cursor.x)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth*2, rectHeight);
		}
		break;
	    case 38: //upArrow pressed
		if (cursor.y>0){
		    cursor.y--;
		    repaint((cursor.x)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth, rectHeight*2);
		}
		break;
	    case 39: //rightArrow
		if (cursor.x<COLS){
		    cursor.x++;
		    repaint((cursor.x-1)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth*2, rectHeight);
		}
		break;
	    case 40: //downArrow
		if (cursor.y<ROWS){
		    cursor.y++;
		    repaint((cursor.x)*rectWidth + xErr, (cursor.y-1)*rectHeight + yErr, rectWidth, rectHeight*2);
		}
		break;
	}
    }

    public void keyReleased(KeyEvent e){}

    //mouseListening stuff comes from here..
    public void mousePressed(MouseEvent e){}
    public void mouseDragged(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}
    public void mouseMoved(MouseEvent e){
	x = e.getX()-xErr;
	y = e.getY()-yErr;
	if (rectWidth>0 && rectHeight>0) {
	    int newx = x/rectWidth;
	    int newy = y/rectHeight;
	    if (((Math.abs(newx - cursor.x) > 0) || (Math.abs(newy - cursor.y) > 0)) ){
		repaint((cursor.x)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth, rectHeight);
		if( newx>=0 && newx<COLS ) 
		    cursor.x = newx;
		else 
		    cursor.x = (newx<0) ? 0 : COLS-1;

		if( newy>=0 && newy<ROWS)
		    cursor.y = newy;
		else
		    cursor.y = (newy<0) ? 0 : ROWS-1;

		//System.out.println("Cursor: " + cursor.x + ", " + cursor.y);
		repaint((cursor.x)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth, rectHeight);
	    }
	}
    }

    public void mouseClicked(MouseEvent e){
    	if(timeToShoot){
	    requestFocus();
	    int res = shipData.shootAt(cursor.x, cursor.y);
	    updateBufImage();
	    repaint();
	    
	    if(res==-1){
    	    	timeToShoot=false;
	    	kala.vpanel.shoot();
	    }
	}
    }

    public void timeToShoot(){
    	this.timeToShoot=true;
    }

    public void mouseEntered(MouseEvent e){
	//requestFocus();
	mouseOver=true;
	repaint((cursor.x)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth, rectHeight);
    }

    public void mouseExited(MouseEvent e){
	mouseOver=false;
	repaint((cursor.x)*rectWidth + xErr, (cursor.y)*rectHeight + yErr, rectWidth, rectHeight);
    }

    //ActionListener
    public void actionPerformed(ActionEvent e){
       System.out.println(e);
    }
}

